/*
 * Created on 2005-6-21
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
import org.jdom.Element;
import java.util.List;
import java.util.Vector;
import java.util.HashMap;
import java.io.*;

/**
 * @author Bin
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class DOMElementComparer {
	final static int SAME = 0;
	final static int CONTAINS = 1;
	final static int CONTAINS_BY = 2;
	final static int OTHER = 3;
	
	final static int FAILURE_INVALID_ENTRY = -1;
	final static int FAILURE_STRUCTURAL = -2;
	final static int FAILURE_NAME = -3;
	final static int FAILURE_VALUE = -4;
	final static int FAILURE_UNKNOWN = -5;

	File dir1, dir2;

	DebugStack debugStack= new DebugStack();
	
	String[] ignoreNames;
	HashMap replaceMap;
	
	DOMElementComparer(String[] ignoreNames,HashMap replaceMap) {
		this(ignoreNames,replaceMap,null,null);
	}
	
	DOMElementComparer(String[] ignoreNames,HashMap replaceMap,File directory1, File directory2){
		this.ignoreNames = ignoreNames;
		this.replaceMap = replaceMap;
		dir1 = directory1;
		dir2 = directory2;
	}
	private boolean isIgnoredName(String nameString){
		if (nameString == null)
			return false;
		if (ignoreNames != null){
			for (int i =0; i< ignoreNames.length;i++){
				if (ignoreNames[i].equals(nameString)) {
					return true;
				}
			}
		}
		return false;
	}
	private String getReplaceString(String originalString) {
		if (originalString == null)
			return null;
		return (String) replaceMap.get(originalString);
	}
	public int isSame(Element e1, Element e2) {
		return isSame(e1, e2, 0);
	}

	private File getFixedFilePath(File dir, String fullFilename){
		if (fullFilename == null || fullFilename.equals("")) return null;
		File file;
		if (dir != null) {
			int index = fullFilename.lastIndexOf(File.separator);
			String filename = fullFilename.substring(index+1);
			file = new File(dir,filename);
		} else {
			file = new File(fullFilename);
		}
		return file;
	}
	
	public int isSame(Element e1, Element e2, int stackLevel){
		if (e1 == null || e2 == null)
			return FAILURE_INVALID_ENTRY;
		/* check if both have the same name */
		String name1=e1.getName();
		String name2=e2.getName();
		//System.out.println(name1+":"+name2);
		if (name1 == null && name2 == null){
		}else if (name1 == null || name2 == null) {
			debugStack.assignMessage(stackLevel, "One of the name is null",FAILURE_NAME);
			return FAILURE_NAME;
		}else if (!name1.equals(name2)) {
			debugStack.assignMessage(stackLevel, name1+"!="+name2,FAILURE_NAME);
			return FAILURE_NAME;
		}else if (name1.equals("Property")) {
			String propertyName1 = e1.getChildText("Name");
			String propertyName2 = e2.getChildText("Name");

			
			if (propertyName1== null || propertyName2 == null){
				debugStack.assignMessage(stackLevel, "Property");
				debugStack.assignMessage(stackLevel+1,propertyName1 +":"+propertyName2,FAILURE_NAME);
				return FAILURE_NAME;
			}else if (propertyName1.equals(propertyName2)) {
				boolean isToIgnoredName = isIgnoredName(propertyName1);
				//System.out.print("Ignore? "+propertyName1+":");
				if (isToIgnoredName == true){
					//System.out.println("TRUE");
					//debugStack.printMessage();
					return SAME;
				} else if (propertyName1.endsWith("_FILE")) {
					String filename1 = e1.getChildText("Value");
					String filename2 = e2.getChildText("Value");
					File file1 = getFixedFilePath(dir1,filename1);
					File file2 = getFixedFilePath(dir2,filename2);
					if (GUITARUtil.hasSameContentFile(file1,file2)==true)
						return SAME;
					else {
						debugStack.assignMessage(stackLevel,"Property");
						debugStack.assignMessage(stackLevel+1,propertyName1 + ":" + propertyName2,FAILURE_VALUE);
						return FAILURE_VALUE;
					}
				} else {
					//System.out.println("FALSE");
				}
			} else {
				debugStack.assignMessage(stackLevel,"Property");
				String propertyValue = e1.getChildText("Value");
				debugStack.assignMessage(stackLevel+1,propertyName1+":"+propertyValue,FAILURE_NAME);
				return FAILURE_NAME;
			}
		}
		
		/* check if both have the same value */
		String value1 = e1.getText().trim();
		String value2 = e2.getText().trim();
		if (value1==null && value2 == null){
			/* both are null, should be fine */
		}
		else if (value1== null || value2 == null){
			debugStack.assignMessage(stackLevel, name1+":"+value1+":"+value2,FAILURE_VALUE);
			return FAILURE_VALUE;
		}
		else if (!value1.equals(value2)) {
			// check to see if can be replaced
			String replace1=getReplaceString(value1);
			String replace2=getReplaceString(value2);
			if (!((replace1!=null) && (replace1.equals(value2))) ||
				((replace2!=null) && (replace2.equals(value1)))) {
				debugStack.assignMessage(stackLevel, value1+":"+value2,FAILURE_VALUE);
				return FAILURE_VALUE;
			}
		}
			
		//System.out.println("Hello");
		/* recursively compare children */
		List children1, children2, attributes1,attributes2;
		children1 = e1.getChildren();
		children2 = e2.getChildren();
		if (children1.size() != children2.size()) {
			debugStack.assignMessage(stackLevel,name1+":"+value1+":diff_children_size",FAILURE_STRUCTURAL);
			return FAILURE_STRUCTURAL;
		}
		int iChildren = children1.size();
		if (value1== null || value1.equals(""))
			debugStack.assignMessage(stackLevel,name1+":"+getTitle(e1));
		else 
			debugStack.assignMessage(stackLevel,name1+":"+value1);
		//debugStack.assignMessage(stackLevel,name1);
		int currentErrorCode = 0;
		for (int i=0; i < iChildren;i++){
			Element child1 = (Element) children1.get(i);
			boolean isFound = false;
			for (int j=0; j< children2.size() && isFound == false; j++){
				Element child2 = (Element) children2.get(j);
				int iDiff = isSame(child1,child2,stackLevel+1);
				if (iDiff==SAME){
					children2.remove(j);
					isFound = true;
				} else if (currentErrorCode > iDiff) {
					currentErrorCode = iDiff;
				}
			}
			if (isFound == false) {
				return currentErrorCode;
			}
		}
		return SAME;
	}
	public String getTitle(Element e) {
		if (e == null)
			return null;
		String name = e.getName().trim();
		if (name == null || name.equals(""))
			return null;
		if (name.equals("Window"))
			return getWindowTitle(e);
		if (name.equals("GUI"))
			return getGUITitle(e);
		if (name.equals("Component"))
			return getComponentTitle(e);
		return null;
	}
	public String getComponentTitle(Element component) {
		if (component == null) 
			return null;
		List propertyList = component.getChildren("Property");
		if (propertyList == null || propertyList.size() == 0)
			return null;
		for (int i = 0; i < propertyList.size();i++) {
			Element property = (Element) propertyList.get(i);
			Element name = property.getChild("Name");
			if (name != null ) {
				String nameText = name.getText().trim();
				if (nameText.equals("Title")) {
					Element value = property.getChild("Value");
					return value.getText().trim();
				}
			}
		}
		return null;
	}
	public String getWindowTitle(Element window) {
		if (window == null) 
			return null;
		List propertyList = window.getChild("Attributes").getChildren("Property");
		if (propertyList == null || propertyList.size() == 0)
			return null;
		for (int i = 0; i < propertyList.size();i++) {
			Element property = (Element) propertyList.get(i);
			Element name = property.getChild("Name");
			if (name != null ) {
				String nameText = name.getText().trim();
				if (nameText.equals("Title")) {
					Element value = property.getChild("Value");
					return value.getText().trim();
				}
			}
		}
		return null;
	}
	public String getGUITitle(Element GUI) {
		if (GUI == null)
			return null;
		Element window = GUI.getChild("Window");
		if (window != null)
			return getWindowTitle(window);
		return null;
	}
	
	public boolean isContain(Element e1, Element e2) {
		return (isSame(e2,union(e1,e2))>=0);
	}

	public void printDebugInfo() {
		debugStack.printMessage();
	}	

	public int getRelation(Element e1, Element e2) {
		if (isSame(e1,e2)>=0)
			return SAME;
		if (isContain(e1,e2))
			return CONTAINS;
		if (isContain(e2,e1))
			return CONTAINS_BY;
		return OTHER;
	}
	
	public Element union(Element e1, Element e2){
		return null;
	}
	
	public static void main(String[] args) {
	}
}
