import java.util.Vector;

public class ErrorSummary
{
	int iMismatchCount;
	int iTitleMismatchCount;
	int iMissingComponentCount;
	int iCorrectCount;
	Vector vErrorLog;
	ErrorSummary()
	{
		vErrorLog = new Vector();
		iMismatchCount=0;
		iMissingComponentCount=0;
		iCorrectCount=0;
		iTitleMismatchCount=0;
	}

	public Vector getErrorLog()
	{
		return vErrorLog;
	}
	public float getErrorScore()
	{
		int errorCount=0;
		errorCount+=iMismatchCount*Constants.REGULAR_WEIGHT;
		errorCount+=iTitleMismatchCount*Constants.TITLE_WEIGHT;
		errorCount+=iMissingComponentCount*Constants.EXTRA_COMPONENT_WEIGHT;

		int overallCount=errorCount+iCorrectCount;
		if (overallCount == 0)
			return 0;
		else 
			return (float)errorCount/(float)overallCount;
	}
	public int getMismatchCount()
	{
		return iMismatchCount;
	}
	public int getMissingComponentCount()
	{
		return iMissingComponentCount;
	}
	public int getCorrectCount()
	{
		return iCorrectCount;
	}
	public int getTitleMismatchCount()
	{
		return iTitleMismatchCount;
	}

	public boolean addErrorStatus(ErrorStatus error)
	{
		boolean isSuccess=false;
		if (error != null)
		{
			int errorCode=error.getErrorCode();
			String errorString = error.getErrorMessage();
			if (errorCode == Constants.SAME)
			{
				iCorrectCount++;
			}
			else
			{
				if (errorCode== Constants.EXTRA_COMPONENT1 ||
					errorCode== Constants.EXTRA_COMPONENT2)
				{
					iMissingComponentCount++;
				}
				else if (errorString != null && errorString.indexOf("->Title=") != -1)
				{
					iTitleMismatchCount++;
				}
				else
				{
					iMismatchCount++;
				}
				vErrorLog.add(error);
			}
			isSuccess=true;
		}
		return isSuccess;
	}

	public boolean addErrorSummary(ErrorSummary summary)
	{
		boolean isSuccess=false;
		if (summary != null)
		{
			float summaryScore=summary.getErrorScore();
			if (summaryScore > 0)
			{
				Vector vErrorSubLog=summary.getErrorLog();
				if (vErrorSubLog != null &&
						vErrorSubLog.size()>0)
				{
					isSuccess=true;
					vErrorLog.addAll(vErrorSubLog);
					iMissingComponentCount+=summary.getMissingComponentCount();
					iCorrectCount+=summary.getCorrectCount();
					iMismatchCount+=summary.getMismatchCount();
				}
			}
			else if (summaryScore == 0)
			{
				isSuccess=true;
			}
		}
		return isSuccess;
	}
}
