import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import java.awt.Container;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JFileChooser;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import javax.swing.JOptionPane;
import java.util.Vector;
import javax.swing.table.TableModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.ScrollPaneConstants;
import java.lang.Integer;
import java.awt.BorderLayout;
import java.awt.Dimension;

public class EventGraphEditor extends JFrame {
	JTable eventGraphTable,infoTable;
	Vector allGUIObjects= null;
	EventGraph eventGraph;
	boolean isGoodEFG=false, isGoodGUI=false;
	File fileEFG,fileGUI,currentDirectory;
	TableModel eventTableModel,infoTableModel;
	

	EventGraphEditor () {
		super("Event Graph Editor");
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				try {
					System.exit(0);
				} catch (Exception e) {
					System.exit(0);
				}
			}
		});

		setSize(900,700);

		JMenuBar jMenubar = new JMenuBar();
		JMenu jMenuFile= new JMenu("File");
		JMenu jMenuSet = new JMenu("Set");

		JMenuItem jMenuItemOpenGUI = new JMenuItem("Select GUI File");
		
		jMenuItemOpenGUI.addActionListener(new ActionListener(){
  			public void actionPerformed(ActionEvent aEvent){
				openGUIMenuAction();
  			}
  		});

		JMenuItem jMenuItemOpenEFG = new JMenuItem("Open Existing Event Graph");
		
		jMenuItemOpenEFG.addActionListener(new ActionListener(){
  			public void actionPerformed(ActionEvent aEvent){
				openEFGMenuAction();
  			}
  		});

		JMenuItem jMenuItemSave = new JMenuItem("Save Event Graph");
		
		jMenuItemSave.addActionListener(new ActionListener(){
  			public void actionPerformed(ActionEvent aEvent){
				saveMenuAction();
  			}
  		});
		JMenuItem jMenuItemSaveAs = new JMenuItem("Save Event Graph As");
		jMenuItemSaveAs.addActionListener(new ActionListener(){
  			public void actionPerformed(ActionEvent aEvent){
				saveAsMenuAction();
  			}
  		});


		JMenuItem jMenuItemSetColumn = new JMenuItem("Column");
		jMenuItemSetColumn.addActionListener(new ActionListener(){
  			public void actionPerformed(ActionEvent aEvent){
				setColumnMenuAction();
  			}
  		});

		JMenuItem jMenuItemSetRow = new JMenuItem("Row");
		jMenuItemSetRow.addActionListener(new ActionListener(){
  			public void actionPerformed(ActionEvent aEvent){
				setRowMenuAction();
  			}
  		});

		jMenubar.add(jMenuFile);
		jMenubar.add(jMenuSet);
		jMenuFile.add(jMenuItemOpenGUI);
		jMenuFile.add(jMenuItemOpenEFG);
		jMenuFile.add(jMenuItemSave);
		jMenuFile.add(jMenuItemSaveAs);
		jMenuSet.add(jMenuItemSetColumn);
		jMenuSet.add(jMenuItemSetRow);

		setJMenuBar(jMenubar);
		show();


	}
	public void openGUIMenuAction() {
		JFileChooser fc = new JFileChooser(currentDirectory);
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			fileGUI = fc.getSelectedFile();
			currentDirectory= fileGUI.getParentFile();
			//This is where a real application would open the file.
			System.out.println("Open GUI: " + fileGUI.getPath() + "." );
			GUIParser guiParser = new GUIParser(fileGUI.getPath(),true,null);
			if (guiParser.isValid()==true) {
				isGoodGUI = true;
				isGoodEFG = false;
				eventGraph = null;
				allGUIObjects = guiParser.getGUIObjects();
				JOptionPane.showMessageDialog(this,
  							"GUI file parsing finished",
							"Finished parsing GUI",
							JOptionPane.INFORMATION_MESSAGE);
			} else {
				JOptionPane.showMessageDialog(this,
  							"GUI file parsing error",
							"Error parsing GUI",
							JOptionPane.ERROR_MESSAGE);
			}
			
		} else {
			System.out.println("Open GUI command cancelled by user." );
		}
	}

	public void openEFGMenuAction() {
		if (isGoodGUI == false) {
			JOptionPane.showMessageDialog(this,
  				"Please select a valid GUI file first",
				"Error: valid GUI file missing",
				JOptionPane.ERROR_MESSAGE);
			return;
		}
		JFileChooser fc = new JFileChooser(currentDirectory);
		int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			fileEFG = fc.getSelectedFile();
			currentDirectory = fileEFG.getParentFile();
			//This is where a real application would open the file.
			System.out.println("Open EFG: " + fileEFG.getName() + "." );
			EventGraphReader eventGraphReader = new EventGraphReader(fileEFG.getPath(),allGUIObjects);
			if (eventGraphReader.isSuccess()== true) {
				isGoodEFG = true;
				eventGraph = eventGraphReader.getEventGraph();
				eventTableModel = new AbstractTableModel() {
          				public int getColumnCount() { return eventGraph.getEGSize()+1; }
          				public int getRowCount() { return eventGraph.getEGSize();}
          				public Object getValueAt(int row, int col) {
						if (col == 0) {
							return new Integer(row);
						} else { 
							return new Integer(eventGraph.getEventGraphStatus(row, col-1));
						} 
					}
					public boolean isCellEditable(int  rowIndex, int columnIndex) {
						if (columnIndex == 0) return false;
						return true;
					}
					public void setValueAt(Object a, int row, int col) {
						if (a instanceof String) {
							try {
								eventGraph.setEventGraphStatus(row, col-1,
									Integer.parseInt((String)a));
							} catch (NumberFormatException e) {
								JOptionPane.showMessageDialog(EventGraphEditor.this,
  									e.toString(),
									"Error Input Integer",
									JOptionPane.ERROR_MESSAGE);
							}
						}
					}	
					public String getColumnName(int columnIndex) {
						if (columnIndex > 0)
							return ""+(columnIndex-1);
						else
							return "";
					}
	
      				};
				eventGraphTable = new JTable(eventTableModel);
				eventGraphTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
				//eventGraphTable.setPreferredScrollableViewportSize(new Dimension(900, 700));

				
				int v=ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED;
				int h=ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED;
				JScrollPane jScrollPane = new JScrollPane(eventGraphTable,v,h);



				infoTableModel = new AbstractTableModel() {
          				public int getColumnCount() { return 4; }
          				public int getRowCount() { return eventGraph.getEGSize();}
          				public Object getValueAt(int row, int col) {
						String returnString;
						GUIEvent event = eventGraph.getEvent(row);
						switch (col) {
							case 0:returnString = ""+row;
								break;
							case 1:returnString = event.getWindow().getPropertyString("Title");
								break;
							case 2:returnString = event.getEventObject().getPropertyString("Title");
								break;
							case 3:returnString = event.getEvent();
								break;
							default:
								returnString = "UNDEFINED";
						} 
						return returnString;
					}
					public String getColumnName(int columnIndex) {
						String returnString;
						switch (columnIndex) {
							case 0:returnString = "";break;
							case 1:returnString = "Window";break;
							case 2:returnString = "Component";break;
							case 3:returnString = "Action";break;
							default: returnString = "UNDEFINED";
						}
						return returnString;
					}
	
      				};
				infoTable = new JTable(infoTableModel);
				infoTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
				//jInfoPane.setPreferredScrollableViewportSize(new Dimension(300, 700));
				JScrollPane jInfoPane = new JScrollPane(infoTable,v,h);
				jInfoPane.setPreferredSize(new Dimension(200,700));

				getContentPane().add(jScrollPane,BorderLayout.CENTER);
				getContentPane().add(jInfoPane, BorderLayout.EAST);
				validate();
				//eventGraphTable.revalidate();
			}
			
		} else {
			System.out.println("Open EFG command cancelled by user." );
		}
	}

	public void saveMenuAction() {
		if (isGoodEFG == false) {
			JOptionPane.showMessageDialog(this,
  				"EFG file is not available",
				"Error: cannot save",
				JOptionPane.ERROR_MESSAGE);
			return;
		}
		JFileChooser fc = new JFileChooser(currentDirectory);
  		if (fileEFG == null) {
  			int returnVal = fc.showSaveDialog(this);
  			if (returnVal == JFileChooser.APPROVE_OPTION) {
  				fileEFG = fc.getSelectedFile();
				currentDirectory = fileEFG.getParentFile();
  				//This is where a real application would open the file.
  				System.out.println("Saving: " + fileEFG.getPath() + "." );
				save(fileEFG);
  			} else {
  				System.out.println("Save command cancelled by user." );
  			}
  		} else {
			save(fileEFG);
  		}
  	}

	public void saveAsMenuAction() {
		if (isGoodEFG == false) {
			JOptionPane.showMessageDialog(this,
  				"EFG file is not available",
				"Error: cannot save",
				JOptionPane.ERROR_MESSAGE);
			return;
		}
		JFileChooser fc = new JFileChooser(currentDirectory);
		int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			fileEFG = fc.getSelectedFile();
			currentDirectory = fileEFG.getParentFile();
			//This is where a real application would open the file.
			System.out.println("Saving: " + fileEFG.getPath() + "." );
			save(fileEFG);
		} else {
			System.out.println("Save command cancelled by user." );
		}
	}

	public void setColumnMenuAction() {
		if (isGoodEFG == false) {
			JOptionPane.showMessageDialog(this,
  				"EFG file is not available",
				"Error: No valid column to work on",
				JOptionPane.ERROR_MESSAGE);
			return;
		}
		JDualOptionDialog dialog = new JDualOptionDialog("Set Column",
					"Column index to change:",
					"Column value to change to:",
					this,
					true);
		if (dialog.isCancelled() == false) {
			try {
				int columnIndex = Integer.parseInt(dialog.getOption1());
				int newValue = Integer.parseInt(dialog.getOption2());

				for (int i = 0; i < eventGraph.getEGSize(); i++) {
					eventGraph.setEventGraphStatus(i,columnIndex,newValue);
				}
				repaint();
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(EventGraphEditor.this,
  							e.toString(),
							"Error Input Integer",
							JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	public void setRowMenuAction() {
		if (isGoodEFG == false) {
			JOptionPane.showMessageDialog(this,
  				"EFG file is not available",
				"Error: No valid row to work on",
				JOptionPane.ERROR_MESSAGE);
			return;
		}
		JDualOptionDialog dialog = new JDualOptionDialog("Set Column",
					"Row index to change:",
					"Row value to change to:",
					this,
					true);
		if (dialog.isCancelled() == false) {
			try {
				int rowIndex = Integer.parseInt(dialog.getOption1());
				int newValue = Integer.parseInt(dialog.getOption2());

				for (int i = 0; i < eventGraph.getEGSize(); i++) {
					eventGraph.setEventGraphStatus(rowIndex,i,newValue);
				}
				repaint();
				
			} catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(EventGraphEditor.this,
  							e.toString(),
							"Error Input Integer",
							JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	public void save(File file) {
		if (eventGraph == null || eventGraph.isValid() == false)
			return;
		if (file == null) return;
		if (file.exists() == false) {
			try {
				file.createNewFile();
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}
		}
		if (file.isFile()) {
			//try {
				EventGraphWriter eventGraphWriter = new EventGraphWriter(eventGraph);
				eventGraphWriter.writeFile(file);
			/*	
			} catch (IOException e) {
				System.out.println("save problem:"+e);
  				JOptionPane.showMessageDialog(this,
  								e.toString(),
								"Error Saving Log",
								JOptionPane.ERROR_MESSAGE);
				e.printStackTrace();
			}*/
		}
	}

	public static void main(String args[]) {
		EventGraphEditor editor = new EventGraphEditor();	
	}

}

