import java.util.Vector;
/*  
 *	Goal: do minimum weight bipartite matching
 *	input graph is left-by-right size matrix with weights
 *	All constants are defined in Constants.java
 */

public class MinimumWeightedBipartiteMatching 
{
	int [][] graph;
	int[] leftMatches;
	int[] rightMatches;
	boolean isSuccessful=false;
	int leftSize=Constants.UNDEFINED, rightSize=Constants.UNDEFINED;

	
	MinimumWeightedBipartiteMatching(int[][] inputGraph)
	{
		graph=inputGraph;
	}

	public boolean match()
	{
		/* taking care of graph correctness */
		if (graph == null)
		{
			isSuccessful=false;
			return isSuccessful;
		}
		leftSize=graph.length;

		if (leftSize==0 || graph[0]==null || graph[0].length ==0 )
		{
			isSuccessful=false;
			return isSuccessful;
		}
		rightSize=graph[0].length;

		for (int i=0; i< leftSize; i++)
		{
			if (graph[i]==null || graph[i].length != rightSize)
			{
				isSuccessful=false;
				return isSuccessful;
			}
		}

		leftMatches=new int[leftSize];
		for (int i=0; i< leftSize; i++)
			leftMatches[i]=Constants.UNDEFINED;
		rightMatches=new int[rightSize];
		for (int j=0; j< leftSize; j++)
			rightMatches[j]=Constants.UNDEFINED;

		int matchCount=0;
		boolean isDone=false;
		int weight=0;

		while (!isDone)
		{
			int rootSide=Constants.LEFT;
			boolean isChanged=false;
			int i=0;
			while(isChanged == false  && i< leftSize)
			{
				if (leftMatches[i] == Constants.UNDEFINED)
				{
					/* build BFT */
					int side=Constants.LEFT;
					BreadthFirstNode root=new BreadthFirstNode(null,
							side,
							i,
							Constants.UNDEFINED
							);
					BreadthFirstTree bfTree= new BreadthFirstTree(
									root,
									leftSize,
									rightSize
									);

					Vector currentLayer = bfTree.getCurrentLayer();
					for (int j=0; j<currentLayer.size();j++)
					{
						BreadthFirstNode node = (BreadthFirstNode)
							currentLayer.elementAt(j);
						int nodeIndex=node.getIndex();
						for (int k=0; k<rightSize; k++)
						{
							if (graph[nodeIndex][k] != Constants.UNDEFINED)
							{
								side=side*Constants.FLIP;
								BreadthFirstNode child = new BreadthFirstNode(
									node,
									side,
									k,
									graph[nodeIndex][k]
									);
								node.addChild(child);
								if (bfTree.addNewLayerNode(child))
								{
									isChanged=true;
								}
							}
						}
					}
				}
				else
				{
				}
				i++;
			}
			if (isChanged == false)
			{
				isDone = true;
			}
		}

		/* now let's work! */
		
		return isSuccessful;
	}

	private int[] breadthFirstSearch()
	{
		return null; 
	}

	public boolean isSuccessful()
	{
		return isSuccessful;
	}

	public int getLeftMatch(int leftSideIndex) 
	{
		if (leftMatches== null || leftSideIndex<0 || leftSideIndex >= leftMatches.length)
			return Constants.UNDEFINED;
		return leftMatches[leftSideIndex];
	}

	public int getRightMatch(int rightSideIndex) 
	{
		if (rightMatches== null || rightSideIndex<0 || rightSideIndex >= rightMatches.length)
			return Constants.UNDEFINED;
		return rightMatches[rightSideIndex];
	}
	public int getEdge(int leftSideIndex, int rightSideIndex)
	{
		if (leftSideIndex < 0 || rightSideIndex < 0 || 
			leftSideIndex >= leftMatches.length || 
			rightSideIndex >= rightMatches.length)
			return Constants.UNDEFINED;
		return graph[leftSideIndex][rightSideIndex];
	}

	
}
