import java.util.Vector;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;
import java.io.File;

public class OracleInfoComparer 
{
	File standardDir, enhancedDir;
	OracleInfoComparer(File dir1, File dir2) 
	{
		standardDir=dir1;
		enhancedDir=dir2;
	}

	public Vector checkSameContainer(
		Vector standardComponentVector, 
		Vector enhancedComponentVector,
		int step,
		int level,
		String path) 
	{
		Vector errorVector=new Vector();

		HashMap componentPartition = new HashMap();

		OracleInfoComponent component;
		String componentTitle;
		 OracleInfoPartition partition;
		for (int i=0; i<standardComponentVector.size();i++)
		{
			component= (OracleInfoComponent) standardComponentVector.elementAt(i);
			componentTitle=component.getStrippedTitle();
			partition = (OracleInfoPartition)
				componentPartition.get(componentTitle);
			if (partition == null)
			{
				partition=new OracleInfoPartition(componentTitle);
			}
			partition.addLeft(component);
		}

		for (int j=0; j<enhancedComponentVector.size();j++)
		{
			component= (OracleInfoComponent) enhancedComponentVector.elementAt(j);
			componentTitle=component.getStrippedTitle();
			partition = (OracleInfoPartition)
				componentPartition.get(componentTitle);
			if (partition == null)
			{
				partition=new OracleInfoPartition(componentTitle);
			}
			partition.addRight(component);
		}

		Set keySet = componentPartition.keySet();
		Iterator keyIterator = keySet.iterator();
		while (keyIterator.hasNext())
		{
			String key=(String)keyIterator.next();
			partition = (OracleInfoPartition)componentPartition.get(key);
			if (partition != null)
			{
				Vector leftVector = partition.getLeftVector();
				Vector rightVector = partition.getRightVector();
				/* assuming Vector cannot be null */
				Vector[][] maskArray = new Vector[leftVector.size()]
								[rightVector.size()];
				int[][] edgeWeights = new int[leftVector.size()]
								[rightVector.size()];
				/* initialize the edge matrix */
				for (int i=0; i < leftVector.size(); i++)
					for (int j=0; j< rightVector.size();j++)
					{
						maskArray[i][j]=null;
						edgeWeights[i][j]=Constants.UNDEFINED;
					}
				/* this two array is used to help reduced the comparison needed */
				int[] forwardMatch=new int[leftVector.size()];
				for (int i=0; i<leftVector.size();i++)
					forwardMatch[i]=Constants.UNDEFINED;
		
				int[] backwardMatch=new int[rightVector.size()];
				for (int j=0; j< rightVector.size(); j++ )
				{
					backwardMatch[j]=Constants.UNDEFINED;
				}
				/* calculate the weight matrix */
				for (int i=0; i< leftVector.size();i++)
				{
					OracleInfoComponent component1=
						(OracleInfoComponent)leftVector.elementAt(i);
					int j=0;
					boolean isFullyMatched=false;
					while(isFullyMatched == false && j<rightVector.size())
					{
						OracleInfoComponent component2=
							(OracleInfoComponent)rightVector.elementAt(j);
						if (backwardMatch[j]==Constants.UNDEFINED)
						{
							maskArray[i][j] = checkSameComponent(component1,component2,step,level);
							if (maskArray[i][j] != null )
								if (maskArray[i][j].size()==0)
								{
									isFullyMatched=true;
									backwardMatch[j]=i;
									forwardMatch[i]=j;
									edgeWeights[i][j]=0;
								}
								else
								{
									edgeWeights[i][j]=
										maskArray[i][j].size();
								}
						}
						j++;
					}
				}
				/* debugging info */
				System.out.println(key);
				for (int i=0; i< leftVector.size();i++)
				{
					for (int j=0; j<rightVector.size();j++)
					{
						if ( maskArray[i][j] ==null)
							System.out.print("E ");
						else
							System.out.print(maskArray[i][j].size()+" ");	
					}
					System.out.println("");
				}
				/* do the matching */
				MinimumWeightedBipartiteMatching match = 
					new MinimumWeightedBipartiteMatching(edgeWeights);
				if (match.match() == false)
				{
					ErrorStatus status=new ErrorStatus(Constants.MATCH_ERROR,
						step,
						level,
						path+"->"+key
					);
					errorVector.add(status);
				}
				else
				{
					/* add all x errors */
					for (int i=0; i< leftVector.size(); i++)
					{
						int k=match.getLeftMatch(i);
						Vector subErrorVector= maskArray[i][k];
						if (subErrorVector==null)
						{
							component= (OracleInfoComponent)
								leftVector.elementAt(i);
							ErrorStatus status=new ErrorStatus(Constants.EXTRA_COMPONENT1,
								step,
								level,
								component.getPath()
							);
							errorVector.add(status);
						}
						else
						{
							errorVector.addAll(subErrorVector);
						}
					}
		
					/* add unmatched y */
					for (int j=0; j< rightVector.size(); j++)
					{
						int k=match.getRightMatch(j);
						if (j==Constants.UNDEFINED)
						{
							component= (OracleInfoComponent)
								rightVector.elementAt(j);
							ErrorStatus status=new ErrorStatus(Constants.EXTRA_COMPONENT2,
								step,
								level,
								component.getPath()
							);
							errorVector.add(status);
						}
					}
				}
			}
		}
		return errorVector;
	}

	private Vector checkSameComponent(
		OracleInfoComponent standardComponent,
		OracleInfoComponent enhancedComponent,
		int step,
		int level)
	{
		if (standardComponent == null || enhancedComponent == null) 
		{
			return null;
		}

		if (standardComponent.getComponentType() != enhancedComponent.getComponentType())
		{
			return null;
		}

		String standardTitle=standardComponent.getPropertyValueString("Title");
		String enhancedTitle=enhancedComponent.getPropertyValueString("Title");

		/* might need to provide translation here */
		if (standardTitle== null || enhancedTitle==null ||
			standardTitle.equals(enhancedTitle))
		{
			return null;
		}

		Vector errorVector = new Vector();
	
		Set standardAttributeKeys = standardComponent.getPropertiesKeySet();
		Set enhancedAttributeKeys = enhancedComponent.getPropertiesKeySet();
		/* check for corresponding keySets? */
		/* UNION */
		Vector keyVector=new Vector();
		Iterator standardKeyIterator=standardAttributeKeys.iterator();
		while (standardKeyIterator.hasNext() == true)
		{
			String standardAttributeName=(String) standardKeyIterator.next();
			if (enhancedAttributeKeys.contains(standardAttributeName))
			{
				keyVector.add(standardAttributeName);
			}
			else
			{
				ErrorStatus status = new ErrorStatus(Constants.ORACLE1_ONLY_KEY,
					step,
					level,
					standardComponent.getPath()+
					"->"+standardAttributeName
					);
				errorVector.add(status);
			}
		}
		if (standardAttributeKeys.size()
			-enhancedAttributeKeys.size()
			-errorVector.size() != 0)
		{
			Iterator enhancedKeyIterator=enhancedAttributeKeys.iterator();
			while (enhancedKeyIterator.hasNext() == true)
			{
				String enhancedAttributeName=(String) enhancedKeyIterator.next();
				if (!standardAttributeKeys.contains(enhancedAttributeName))
				{
					ErrorStatus status = new ErrorStatus(Constants.ORACLE2_ONLY_KEY,
						step,
						level,
						enhancedComponent.getPath()+
						"->"+enhancedAttributeName
					);
					errorVector.add(status);
				}
			}
		}

		for(int i=0; i<keyVector.size();i++)
		{
			String standardAttributeName=
			(String) keyVector.elementAt(i);

			String standardAttributeValueString =
				(String) standardComponent.getPropertyValueString
					(standardAttributeName,null);

			String enhancedAttributeValueString =
				(String) enhancedComponent.getPropertyValueString
					(standardAttributeName,null);
			//System.out.println(standardAttributeValueString+":"+
				//enhancedAttributeValueString);
			if (standardAttributeValueString == null)
			{
				ErrorStatus status = new ErrorStatus(Constants.ERROR_NULL1,
					step,
					level,
					standardComponent.getPath()+
					"->"+standardAttributeName+"="+
					" Missing value in oracle 1"
					);
				errorVector.add(status);
			}
			else if (enhancedAttributeValueString==null)
			{
				ErrorStatus status = new ErrorStatus(Constants.ERROR_NULL2,
					step,
					level,
					enhancedComponent.getPath()+
					"->"+standardAttributeName+"="+
					" Missing value in oracle 2"
					);
				errorVector.add(status);
			}
			else if (!standardAttributeValueString.equals(
				enhancedAttributeValueString))
			{
				ErrorStatus status = new ErrorStatus(Constants.ERROR_VALUE,
					step,
					level,
					standardComponent.getPath()+
					"->"+standardAttributeName+"="+
					standardAttributeValueString+"|"+
					enhancedAttributeValueString
					);
				errorVector.add(status);
			}
		}
		if (standardComponent.getComponentType() == OracleInfoComponent.GUI)
		{
			errorVector.addAll(checkSameContainer(
				standardComponent.getChildrenVector(),
				enhancedComponent.getChildrenVector(),
				step,
				level,
				standardComponent.getPath()
			));
		}
		return errorVector;
	}
}
