import java.util.Vector;
import java.util.Set;
import java.util.Iterator;
import java.io.File;

public class OracleInfoComparerSimple 
{
	File standardDir, enhancedDir;
	String[] ignoreList;
	Vector knownErrorVector;
	OracleInfoComparerSimple(String[] ignoreNames,Vector knownErrorVector,File dir1, File dir2) 
	{
		standardDir=dir1;
		enhancedDir=dir2;
		ignoreList=ignoreNames;
		this.knownErrorVector=knownErrorVector;
	}

	public ErrorSummary checkSameContainer(
		Vector standardComponentVector, 
		Vector enhancedComponentVector,
		int step,
		int level) 
	{
/*
		int standardSize=standardComponentVector.size();
		int enhancedSize=enhancedComponentVector.size();
		int sizeDiff = Math.abs(standardSize - enhancedSize);

		if (sizeDiff > 30 && 
			(((float) sizeDiff)/standardSize >0.5 ||
			((float) sizeDiff)/enhancedSize >0.5 ))
		{
			return null;
		}
*/
		ErrorSummary summary=new ErrorSummary();
		ErrorStatus status=null;

		ErrorSummary[][] maskArray= new ErrorSummary[standardComponentVector.size()]
					[enhancedComponentVector.size()];
		for (int i=0; i<standardComponentVector.size();i++)
			for (int j=0; j<enhancedComponentVector.size();j++)
				maskArray[i][j]=null;

		int[] forwardMatch=new int[standardComponentVector.size()];
		for (int i=0; i<standardComponentVector.size();i++)
			forwardMatch[i]=Constants.UNDEFINED;

		int[] backwardMatch=new int[enhancedComponentVector.size()];
		for (int j=0; j< enhancedComponentVector.size(); j++ )
		{
			backwardMatch[j]=Constants.UNDEFINED;
		}

		/* create the error vector array for each pair of component */
		for (int i=0; i< standardComponentVector.size();i++)
		{
			OracleInfoComponent component1=
				(OracleInfoComponent)standardComponentVector.elementAt(i);
			status=new ErrorStatus(Constants.EXTRA_COMPONENT1,-1,-1,component1.getPath());
			/* actively ignore components */
			if (checkExpectedError(status))
			{
				continue;
			}
			int j=0;
			boolean isFullyMatched=false;
			while(isFullyMatched == false && j<enhancedComponentVector.size())
			{
				OracleInfoComponent component2=
					(OracleInfoComponent)enhancedComponentVector.elementAt(j);
				/*actively ignore components */
				status=new ErrorStatus(Constants.EXTRA_COMPONENT2,-1,-1,
									component2.getPath());
				if (checkExpectedError(status))
				{
					j++;
					continue;
				}
				if (backwardMatch[j]==Constants.UNDEFINED)
				{
					maskArray[i][j] = checkSameComponent(component1,component2,step,level);
					if (maskArray[i][j] != null && maskArray[i][j].getErrorScore()==0)
					{
						isFullyMatched=true;
						backwardMatch[j]=i;
						forwardMatch[i]=j;
					}
				}
				j++;
			}
		}
		if (System.getProperty("Debug") != null){
			System.out.println(standardComponentVector.size()+"x"+enhancedComponentVector.size());
			for (int i=0; i< standardComponentVector.size();i++)
			{
				for (int j=0; j<enhancedComponentVector.size();j++)
				{
					if ( maskArray[i][j] ==null)
						System.out.print("E ");
					else
						System.out.print(maskArray[i][j].getErrorScore()+" ");	
				}
				System.out.println("");
			}
		}

		/* greedy */
		boolean isDone=false;
		while (!isDone)
		{
			int minForwardIndex=Constants.UNDEFINED;
			int minBackwardIndex=Constants.UNDEFINED;
			float minScore = Constants.UNDEFINED;
			for (int i=0; i< forwardMatch.length; i++)
			{
				if (forwardMatch[i] == Constants.UNDEFINED)
				{
					for (int j=0; j<backwardMatch.length; j++)
					{
						if (backwardMatch[j]==Constants.UNDEFINED && 
							maskArray[i][j] != null)
						{
							
							if (minForwardIndex==Constants.UNDEFINED ||
								minScore>maskArray[i][j].getErrorScore())
							{
								minForwardIndex=i;
								minBackwardIndex=j;
								minScore=maskArray[i][j].getErrorScore();
							}
						}
					}
				}
			}
			if (minScore == Constants.UNDEFINED)
			{
				isDone=true;
			}
			else
			{
				forwardMatch[minForwardIndex]=minBackwardIndex;
				backwardMatch[minBackwardIndex]=minForwardIndex;
				summary.addErrorSummary(maskArray[minForwardIndex][minBackwardIndex]);
			}
		}
		if (forwardMatch.length > backwardMatch.length)
		{
			for (int i=0; i< forwardMatch.length; i++)
			{
				if (forwardMatch[i]==Constants.UNDEFINED)
				{
					OracleInfoComponent component=
						(OracleInfoComponent)standardComponentVector.elementAt(i);
					status=new ErrorStatus(Constants.EXTRA_COMPONENT1,
						step,
						level,
						component.getPath()
						);
					if (checkExpectedError(status)==false) 
						summary.addErrorStatus(status);
				}
			}
		}
		else
		{
			for (int j=0; j< backwardMatch.length; j++)
			{
				if (backwardMatch[j]==Constants.UNDEFINED)
				{
					OracleInfoComponent component=
						(OracleInfoComponent)enhancedComponentVector.elementAt(j);
					status=new ErrorStatus(Constants.EXTRA_COMPONENT2,
						step,
						level,
						component.getPath()
						);
					if (checkExpectedError(status)==false) 
						summary.addErrorStatus(status);
				}
			}
		}
		if (System.getProperty("Debug") != null){
			System.out.print("=>"+summary.getMismatchCount());
			System.out.print("+"+summary.getMissingComponentCount());
			System.out.print("|"+summary.getCorrectCount());
			System.out.println("="+summary.getErrorScore());
		}
		
		return summary;
	}

	private boolean checkExpectedError(ErrorStatus error)
	{
		if (knownErrorVector == null || error == null)
			return false;
		String errorString="";
		errorString+=error.getErrorCode()+",-1,-1,"+error.getErrorMessage();
		if (System.getProperty("Debug") != null)
		{
			System.out.println("CheckExpectedError:"+errorString);
		}

		if (knownErrorVector.contains(errorString))
			return true;
		return false;
	}

	private ErrorSummary checkSameComponent(
		OracleInfoComponent standardComponent,
		OracleInfoComponent enhancedComponent,
		int step,
		int level)
	{
		ErrorSummary summary= new ErrorSummary();
		if (standardComponent == null || enhancedComponent == null) 
		{
			return null;
		}

		if (standardComponent.getComponentType() != enhancedComponent.getComponentType())
		{
			return null;
		}

		/* might need to provide translation here */
		/*
		String standardTitle=standardComponent.getPropertyValueString("Title");
		String enhancedTitle=enhancedComponent.getPropertyValueString("Title");

		if (standardTitle== null || enhancedTitle==null ||
			standardTitle.equals(enhancedTitle))
		{
			return null;
		}
		*/
	
		Set standardAttributeKeys = standardComponent.getPropertiesKeySet();
		Set enhancedAttributeKeys = enhancedComponent.getPropertiesKeySet();
		/* check for corresponding keySets? */
		/* UNION */
		Vector keyVector=new Vector();
		Iterator standardKeyIterator=standardAttributeKeys.iterator();
		int iExtraStandardKey=0;
		while (standardKeyIterator.hasNext() == true)
		{
			String standardAttributeName=(String) standardKeyIterator.next();
			if (enhancedAttributeKeys.contains(standardAttributeName))
			{
				keyVector.add(standardAttributeName);
			}
			else
			{
				ErrorStatus status = new ErrorStatus(Constants.ORACLE1_ONLY_KEY,
					step,
					level,
					standardComponent.getPath()+
					"->"+standardAttributeName
					);
				if (checkExpectedError(status)==false)
				{
					summary.addErrorStatus(status);
					iExtraStandardKey++;
				}
			}
		}
		if (standardAttributeKeys.size()
			-enhancedAttributeKeys.size()
			-iExtraStandardKey != 0)
		{
			Iterator enhancedKeyIterator=enhancedAttributeKeys.iterator();
			while (enhancedKeyIterator.hasNext() == true)
			{
				String enhancedAttributeName=(String) enhancedKeyIterator.next();
				if (!standardAttributeKeys.contains(enhancedAttributeName))
				{
					ErrorStatus status = new ErrorStatus(Constants.ORACLE2_ONLY_KEY,
						step,
						level,
						enhancedComponent.getPath()+
						"->"+enhancedAttributeName
					);
					if (checkExpectedError(status)==false)
						summary.addErrorStatus(status);
				}
			}
		}

		for(int i=0; i<keyVector.size();i++)
		{
			String sharedAttributeName=
				(String) keyVector.elementAt(i);
			if (ignoreList != null)
			{
				int k=0;
				while (k< ignoreList.length &&
					!ignoreList[k].equals(sharedAttributeName))
				{
					k++;
				}
				if (k != ignoreList.length)
				{
					summary.addErrorStatus(new ErrorStatus(Constants.SAME));
					continue;
				}
			}
			
			String standardAttributeValueString;
			String enhancedAttributeValueString;

			if (sharedAttributeName.equals("Title"))
			{
				standardAttributeValueString =
					standardComponent.getStrippedTitle();
				enhancedAttributeValueString =
					enhancedComponent.getStrippedTitle();
			}
			else
			{
				standardAttributeValueString =
					(String) standardComponent.getPropertyValueString
						(sharedAttributeName,null);

				enhancedAttributeValueString =
					(String) enhancedComponent.getPropertyValueString
						(sharedAttributeName,null);
			}

			if ((sharedAttributeName.equals("X") ||
				sharedAttributeName.equals("Y")) &&
				standardComponent.getComponentType()==OracleInfoComponent.GUI)
			{
				summary.addErrorStatus(new ErrorStatus(Constants.SAME));	
				continue;
			} 
					

			//System.out.println(standardAttributeValueString+":"+
				//enhancedAttributeValueString);
			if (standardAttributeValueString == null)
			{
				ErrorStatus status = new ErrorStatus(Constants.ERROR_NULL1,
					step,
					level,
					standardComponent.getPath()+
					"->"+sharedAttributeName+"="+
					" Missing value in oracle 1"
					);
				summary.addErrorStatus(status);
			}
			else if (enhancedAttributeValueString==null)
			{
				ErrorStatus status = new ErrorStatus(Constants.ERROR_NULL2,
					step,
					level,
					enhancedComponent.getPath()+
					"->"+sharedAttributeName+"="+
					" Missing value in oracle 2"
					);
				summary.addErrorStatus(status);
			}
			else if (!standardAttributeValueString.equals(
				enhancedAttributeValueString))
			{
				ErrorStatus status = new ErrorStatus(Constants.ERROR_VALUE,
					step,
					level,
					standardComponent.getPath()+
					"->"+sharedAttributeName+"="+
					standardAttributeValueString+"|"+
					enhancedAttributeValueString
					);
				if (checkExpectedError(status)==false)
					summary.addErrorStatus(status);
			}
			else
			{
				summary.addErrorStatus(new ErrorStatus(Constants.SAME));
			}
		}
		if (standardComponent.getComponentType() == OracleInfoComponent.GUI)
		{
			ErrorSummary subErrorSummary = checkSameContainer(
				standardComponent.getChildrenVector(),
				enhancedComponent.getChildrenVector(),
				step,
				level
			);
			if (subErrorSummary != null)
			{
				summary.addErrorSummary(subErrorSummary);
			}
			else
			{
				return null;
			}
		}
		return summary;
	}
}
