import java.util.HashMap;
import java.util.Vector;
import java.util.Set;

public class OracleInfoComponent {
	final static int GUI=1;
	final static int Component=2;
	final static int Menu=3;
	int type;
	HashMap propertiesMap;
	Vector childrenVector;
	OracleInfoComponent parent=null;

	OracleInfoComponent(int componentType) {
		type=componentType;
		propertiesMap=new HashMap();
		childrenVector = new Vector();
	}

	public void addProperty(String propertyName, Vector propertyValues){
		propertiesMap.put(propertyName,propertyValues);
	}

	public Vector getPropertyValue(String propertyName) {
		return (Vector) propertiesMap.get(propertyName);
	}

	public String getPropertyValueString(String propertyName,String separateString) {
		String separator="_";
		if (separateString != null) 
			separator = separateString;
		Vector propertyValueVector= (Vector) propertiesMap.get(propertyName);
		if (propertyValueVector == null)
			return null;
		if (propertyValueVector.size() == 0)
			return "";

		String returnString = (String) propertyValueVector.elementAt(0);
		for (int i =1; i < propertyValueVector.size(); i++) {
			returnString = returnString + separator + (String) propertyValueVector.elementAt(i);
		}
		return returnString;
	}

	public Set getPropertiesKeySet() {
		return propertiesMap.keySet();
	}

	public int getComponentType() {
		return type;
	}

	public Vector getChildrenVector() {
		return childrenVector;
	}

	public boolean addChild(OracleInfoComponent component) {
		if (childrenVector == null)
			return false;
		childrenVector.add(component);
		return true;
	}

	public boolean addParent(OracleInfoComponent parent) {
		if (type==GUI)
			return false;
		this.parent=parent;
		return true;
	}
	public OracleInfoComponent getParent() {
		return parent;
	}

	public String getPropertyValueString(String propertyName) {
		Vector valueVector = (Vector) propertiesMap.get(propertyName);
		if (valueVector != null && valueVector.size() !=0) {
			String returnString = (String) valueVector.elementAt(0);

			for (int i =1; i < valueVector.size(); i++) {
				returnString = returnString +","+(String) valueVector.elementAt(i);
			}
			return returnString;
		}
		return null;
	}
/*
	public String getPath() {
		if (type==GUI)
			return getPropertyValueString("Title");
		else
			return getParent().getPropertyValueString("Title")+
				"->"+getPropertyValueString("Title");
	}
*/

	public String getPath() {
		if (type==GUI)
			return getStrippedTitle();
		else
			return getParent().getStrippedTitle()+
				"->"+getPropertyValueString("Title");
	}
	public String getStrippedTitle() 
	{
		String title=getPropertyValueString("Title");
		if (System.getProperty("DebugTitle") != null)
		{
			System.out.println("Stripping title "+title);
		}
		/* trying to remove the index number here */
		int index = title.lastIndexOf("_");
		if (index != -1)
		{
			boolean isDisidentified=false;
			int i=index+1;
			while (isDisidentified == false && 
				i< title.length())
			{
				if (title.charAt(i) > '9' || 
					title.charAt(i) < '0')
				{
					isDisidentified=true;
				}
				i++;
			}
			if (isDisidentified==false)
			{
				if (index > 0)
					title = title.substring(0,index);
				else
					title = "NULL";
			}
		}

		/* trying to remove the _N_ or _R_ here */
		if (title.endsWith("_N") || title.endsWith("_R"))
		{
			title=title.substring(0,title.length()-2);
		}
		return title;
	}

	public void print() {
		switch (type) {
			case GUI:
				System.out.println("GUI");
				System.out.println("Title "+getPropertyValueString("Title"));
				break;
			case Component:
				System.out.println("\tComponent");
				System.out.println("\tTitle "+getPropertyValueString("Title"));
				break;
			case Menu:
				System.out.println("Menu");
				break;
			default:
				break;
		}


		for (int i=0; i< childrenVector.size(); i++) {
			OracleInfoComponent component = (OracleInfoComponent) childrenVector.elementAt(i);
			component.print();
		}
		System.out.println("");
		
	}
	
}
