import org.jdom.Element;
import java.util.List;
import java.util.Vector;
import java.util.HashMap;
import java.io.*;

public class OracleInfoParser {
boolean isValid = false;
Vector windowVector;
Vector componentVector;

	OracleInfoParser(Element oracleInfo) {
		if (oracleInfo == null)
			return;
		if (!oracleInfo.getName().equals("Oracleinfo")) {
			System.out.println("Error:"+ oracleInfo.getName()+" is not legal");
			return;
		}
		List GUIList = oracleInfo.getChildren("GUI");
		if (GUIList.size() < 0 ) {
			System.out.println("Error: no GUI in oracleInfo");
			return;
		}
		windowVector = new Vector();
		componentVector = new Vector();
		for (int i =0; i < GUIList.size(); i++) {
			OracleInfoComponent guiComponent = new OracleInfoComponent(OracleInfoComponent.GUI);
			if (parseGUI(guiComponent,(Element) GUIList.get(i)) == false ) {
				System.out.println("Error: parsing GUI number " + i);
				return;
			}
			windowVector.add(guiComponent);
		}
		isValid=true;
		
	}

	private boolean parseGUI(OracleInfoComponent guiComponent, Element guiElement) {
		Element property;
		String propertyName, propertyValue;


		Element window = guiElement.getChild("Window");
		if (window==null) {
			Vector titleVector = new Vector();
			titleVector.add("NULL_TITLE");
			guiComponent.addProperty("Title",titleVector);
		} else {
			List propertyList = window.getChild("Attributes").getChildren("Property");
			for (int i=0; i< propertyList.size();i++){
				property = (Element) propertyList.get(i);
				//System.out.println("BBBGGG"+property.getName());
				propertyName = property.getChildText("Name");
				List propertyValueList = property.getChildren("Value");
				Vector propertyValueVector = new Vector();
				for (int j=0; j< propertyValueList.size();j++){
					propertyValue = ((Element)propertyValueList.get(j)).getValue();
					propertyValueVector.add(propertyValue);
				}
				guiComponent.addProperty(propertyName,propertyValueVector);
			}
		}

		List componentList = guiElement.getChildren("Component");
		for (int i = 0; i< componentList.size(); i++ ) {
			OracleInfoComponent component=new OracleInfoComponent(OracleInfoComponent.Component);
			if (parseComponent(component,(Element) componentList.get(i)) == false) {
				System.out.println("Error: parseing component number "+ i);
				System.out.println("\t from "+guiComponent.getPropertyValueString("Title"));
				return false;
			}
			componentVector.add(component);
			guiComponent.addChild(component);
			component.addParent(guiComponent);
		}
		return true;
	}

	private boolean parseComponent(OracleInfoComponent component, Element componentElement) {
		Element property;
		String propertyName, propertyValue;
		List propertyList = componentElement.getChildren("Property");

		for (int i=0; i< propertyList.size();i++){
			property = (Element) propertyList.get(i);
			//System.out.println("BBBGGG"+property.getName());
			propertyName = property.getChildText("Name");
			List propertyValueList = property.getChildren("Value");
			Vector propertyValueVector = new Vector();
			for (int j=0; j< propertyValueList.size();j++){
				propertyValue = ((Element)propertyValueList.get(j)).getValue();
				propertyValueVector.add(propertyValue);
			}
			component.addProperty(propertyName,propertyValueVector);
		}
		return true;
	}

	public boolean isValid() {
		return isValid;
	}

	public Vector getWindows() {
		if (isValid == true)
			return windowVector;
		return null;
	}

	public Vector getComponent() {
		if (isValid == true)
			return componentVector;
		return null;
	}

	public static void usage() {
		System.out.println("Usage: java OracleInfoParser <oracle file> <oracle number>");
		System.out.println("\t<oracle number> is between 0 and # of oralceInfo in the oracle file");
		System.exit(0);
	}
	public static void main(String args[]) {

		if (args.length != 2) {
			usage();
		}

		try {
			org.jdom.input.SAXBuilder builder = new org.jdom.input.SAXBuilder();
			org.jdom.Document doc = builder.build(new File(args[0]));
			Element root = doc.getRootElement();
			List oracleInfoList = root.getChild("Testcase").getChildren("Oracleinfo");

			int iOracleInfoRequested = Integer.parseInt(args[1]);
			if ( iOracleInfoRequested < 0 ) {
				usage();
			} else if (oracleInfoList.size() <= iOracleInfoRequested) {
				System.out.println("Total OralceInfo "+oracleInfoList.size());
				usage();
			}else {
				Element OracleInfoElement = (Element) oracleInfoList.get(iOracleInfoRequested);
				OracleInfoParser parser = new OracleInfoParser(OracleInfoElement);
				if (parser.isValid() == false) {
					System.out.println("Error: Parsing requested oracleInfo");
					System.exit(0);
				}

				Vector guiVector=parser.getWindows();
				for (int i = 0; i < guiVector.size(); i++ ) {
					OracleInfoComponent gui = (OracleInfoComponent) guiVector.elementAt(i);
					gui.print();
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
