import java.io.File;
import java.util.Vector;

public class StaticGUIAnalyzer
{
	String guiFilename1, guiFilename2;
	boolean isRelative;
	ErrorSummary summary;

	StaticGUIAnalyzer(String guiFilename1, String guiFilename2, boolean isRelative)
	{
		this.guiFilename1=guiFilename1;
		this.guiFilename2=guiFilename2;
		this.isRelative=isRelative;
	}

	public boolean analyze()
	{
		boolean isSuccess=true;
		GUIParser parser1= new GUIParser(
							guiFilename1,
							false,
							null
							);
		Vector allGUIObject1 = parser1.getGUIObjects();

			
		GUIToOracleInfoConvertor convertor1 = new GUIToOracleInfoConvertor(allGUIObject1);
		if (convertor1.isValid() == false) {
			System.out.println("Error: Converting failure 1");
			System.exit(0);
		}

		GUIParser parser2=new GUIParser(
							guiFilename2,
							false,
							null
							);
		Vector allGUIObject2 = parser2.getGUIObjects();
		GUIToOracleInfoConvertor convertor2 = new GUIToOracleInfoConvertor(allGUIObject2);
		if (convertor2.isValid() == false) {
			System.out.println("Error: Converting failure 2");
			System.exit(0);
		}

		OracleInfoComparerSimple comparer=null;

		if (isRelative=true)
		{
			try
			{
				File guiFile1=new File(guiFilename1);
				if (guiFile1 == null)
				{
					return false;
				}
				File dir1 = guiFile1.getParentFile();
				
				File guiFile2=new File(guiFilename2);
				if (guiFile2 == null)
				{
					return false;
				}
				File dir2 = guiFile1.getParentFile();
				
				comparer = new OracleInfoComparerSimple(null,null,dir1,dir2);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
		else
		{
			comparer = new OracleInfoComparerSimple(null,null,null,null);
		}

		if (comparer == null)
			return false;

		summary=comparer.checkSameContainer(convertor1.getWindows(),
												convertor2.getWindows(),
												-1, -1);
		if (summary != null)
		{
			isSuccess=true;
		}
		else
		{
			isSuccess=false;
		}

		return isSuccess;
	}

	public void printResultSimple()
	{
		if (summary==null)
		{
			System.out.println("Error: null error summary @ printResultSimple");
			return;
		}
		if (summary.getErrorScore()==0)
		{
			System.out.println("0,");
			return;
		}
		
		Vector errorVector = summary.getErrorLog();
		for (int i =0; i<errorVector.size(); i++)
		{
			ErrorStatus errorStatus = (ErrorStatus) errorVector.elementAt(i);
			int dErrorCode = errorStatus.getErrorCode();
			if (dErrorCode >=0) {
				System.out.println(dErrorCode+",");
			} else {
				System.out.print(dErrorCode+","+errorStatus.getErrorStage()); 
				System.out.print(","+errorStatus.getErrorLevel()+",");
				System.out.println(errorStatus.getErrorMessage());
				//printDebugInfo();
			}
		}

	}

	public static void printUsage()
	{
		System.out.println("\nUsage:");
		System.out.println("java StaticGUIAnalyzer [options] <GUI File 1> <GUI File 2>");
		System.out.println("\nAvailable options:");
		System.out.println("\t-r|--relativePath");
		System.exit(1);
	}

	public static void main(String[] args) 
	{
		CmdLineParser cmdLineParser = new CmdLineParser();
		CmdLineParser.Option oRelativePath = cmdLineParser.addBooleanOption('r',"relativePath");
		try
		{
			cmdLineParser.parse(args);
		}
		catch(CmdLineParser.OptionException e)
		{
			printUsage();
		}

		Boolean isBRelative = (Boolean) cmdLineParser.getOptionValue(oRelativePath);

		boolean isRelative;
		if (isBRelative == null || isBRelative.booleanValue()==false)
			isRelative=false;
		else
			isRelative=true;

		String [] guiFilenames = cmdLineParser.getRemainingArgs();
		if (guiFilenames.length != 2)
		{
			printUsage();
		}

		StaticGUIAnalyzer analyzer = new StaticGUIAnalyzer(
							guiFilenames[0],
							guiFilenames[1],
							isRelative);

		analyzer.analyze();
		analyzer.printResultSimple();
	}
}
